/**
  * Lägger till knappar för att filtrera metodik.
  *
  * Skapar knappar för att välja att visa eller dölja metoder och tips för
  * specifika operativsystem (iOS, Android) eller dokumentformat (PDF, Word).
  *
  * Filtreringen sker genom att sätta en klass på `body`-elementet, som triggar
  * CSS-regler för vad som ska visas eller döljas.
  */
((doc) => {
    "use strict";


    /**
     * Filterknapp.
     *
     * Knappen sätter klasser på body-elementet för att via CSS visa eller
     * dölja element med vissa klasser.
     *
     * Den har initialt `aria-pressed=true`. Värdet växlar mellan true och false
     * för varje tryck på knappen, och indikerar om det innehåll knappen
     * represeterar är synligt (det vill säga att motsvarande filterklass
     * INTE är satt på `body`).
     */
    class FilterButton {

        /**
         * Konstruera en knappinstans.
         *
         * Skapa och infoga ett `button`-element.
         *
         * @param parent Föräldraelement i DOM-trädet.
         * @param label  Knapptext.
         * @param filter Filterklass för body.
         * @param lang   BCP 47 språkkod för främmande språk.
         */
        constructor (parent, label, filter, lang = null) {
            this._button = parent.appendChild(doc.createElement("button"));
            this._button.type      = "button";
            this._button.className = "secondary";

            if (null !== lang) {
                this._button.lang = lang;
            }

            this._button.setAttribute("aria-pressed", "true");

            // Span för ikon, dolt för hjälpmedel
            this._button.appendChild(doc.createElement("span")).
                setAttribute("aria-hidden", "true");

            // Knapptext
            this._button.appendChild(doc.createTextNode(label));

            this._button.addEventListener("click", (event) => {
                this.toggle(event, filter);
            });
        }

        /**
         * Växla knappens tillstånd mellan "nedtryckt" och "uppsläppt".
         * innehåll med en specifik klass.
         *
         * @param {UIEvent} event  Händelseinformation.
         * @param {string} filter Filterklass för body.
         */
        toggle (event, filter) {
            event.stopPropagation();
            event.preventDefault();
            const pressed = "true" === this._button.getAttribute("aria-pressed");
            if (pressed) {
                doc.body.classList.add(filter);
                this._button.setAttribute("aria-pressed", "false");
            } else {
                doc.body.classList.remove(filter);
                this._button.setAttribute("aria-pressed", "true");
            }
        }

    }


    /**
     * Skapa filterknappar för appgranskning.
     *
     * Skapa knappar för att filtrera på Android eller iOS.
     *
     * @param {HTMLElement} parent Föräldraelement i DOM-trädet.
     */
    function createAppFilter(parent) {
        new FilterButton(parent, "Android", "hide-android", "en");
        new FilterButton(parent, "iOS",     "hide-ios");
    }


    /**
     * Skapa filterknappar för dokumentgranskning.
     *
     * Skapa knappar för att filtrera på PDF eller Word.
     *
     * @param {HTMLElement} parent Föräldraelement i DOM-trädet.
     */
    function createDocFilter(parent) {
        new FilterButton(parent, "PDF",  "hide-pdf");
        new FilterButton(parent, "Word", "hide-word", "en");
    }

    /**
     * Skapa filterknapp för otaggade PDF-dokument.
     *
     * Skapa en knapp för att filtrera innehåll som inte är relevant vid
     * granskning av otaggade PDF-dokument.
     *
     * @param {HTMLElement} parent Föräldraelement i DOM-trädet.
     */
    function createUntaggedPdfFilter (parent) {
        new FilterButton(parent, "Taggad PDF", "hide-pdf-untagged");
    }


    /*
     * Initiering.
     */

    // Kontrollera kompatibilitet
    if ("undefined" === typeof doc.body.classList) {
        return;
    }

    // Rubrik och behållare
    const parent = doc.getElementById("intro-section");
    parent.appendChild(doc.createElement("h3"))
    .appendChild(doc.createTextNode("Filter"));

    const p     = parent.appendChild(doc.createElement("p"));
    p.className = "filter";
    p.appendChild(doc.createTextNode("Visa metodik för "));

    // Skapa filterknappar, beroende på granskningsdokument
    switch (doc.body.dataset.filters) {
        case "app":
            createAppFilter(p);
            break;

        case "doc":
            createDocFilter(p);
            break;

        default:
            break;
    }

    // Filterknapp för otaggade PDF-dokument.
    const placeholder = doc.getElementById("pdf-untagged-placeholder");
    if (placeholder) {
        const p = doc.createElement("p");
        p.className = "filter filter-pdf";
        p.appendChild(doc.createTextNode("Visa relevanta kontroller för "));
        createUntaggedPdfFilter(p);
        placeholder.parentNode.replaceChild(p, placeholder);
    }
})(document);
